﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using SYSCTL;

namespace DeviceApplication1
{
    public partial class Form1 : Form
    {
        DeviceInfo DevInfo;
        KeypadControl KBD_CTL;
        FunctionKeyControl FnKeyCTL;
        BacklightControl BKL_CTL;
        WLANControl WLAN_CTL;
        BluetoothControl BT_CTL;
        //GPRSControl GPRS_CTL;
        //TouchPanelControl TP_CTL;
        SystemControl SYS_CTL;
        Boot Boot_CTL;

        #region Arrays

        VKCode[] VKCodeList =
        {
            VKCode.KEY_0,
            VKCode.KEY_1,
            VKCode.KEY_2,
            VKCode.KEY_3,
            VKCode.KEY_4,
            VKCode.KEY_5,
            VKCode.KEY_6,
            VKCode.KEY_7,
            VKCode.KEY_8,
            VKCode.KEY_9,

            VKCode.TAB_KEY,
            VKCode.SPACE_KEY,
            VKCode.ESC_KEY,
            VKCode.BACK_KEY,
            VKCode.ENTER_KEY,
            VKCode.UP_KEY,
            VKCode.DOWN_KEY,
            VKCode.LEFT_KEY,
            VKCode.RIGHT_KEY,
            VKCode.DOT_KEY,

            VKCode.SCAN_KEY,
            VKCode.FN_KEY,
            VKCode.COMBO_KEY,
            VKCode.POWER_KEY
        };

        FnVKCode[] FnVKCodeList =
        {
            FnVKCode.F1_Key,
            FnVKCode.F2_Key,
            FnVKCode.F3_Key,
            FnVKCode.F4_Key
        };

        BKLMode[] BKLModeList = 
        {
            BKLMode.OFF,
            BKLMode.ALWAYS_ON,
            BKLMode.ENABLE_TIMER
        };

        BKLTimer[] BKLTimerList = 
        {
            BKLTimer.TIMER_15_SEC,
            BKLTimer.TIMER_30_SEC,
            BKLTimer.TIMER_45_SEC,
            BKLTimer.TIMER_1_MIN,
            BKLTimer.TIMER_3_MIN,
            BKLTimer.TIMER_5_MIN,
            BKLTimer.TIMER_10_MIN,
            BKLTimer.TIMER_15_MIN
        };

        ACPowerScheme[] ACPowerSchemeList =
        {
            ACPowerScheme.SCHEME_1_MIN,
            ACPowerScheme.SCHEME_2_MIN,
            ACPowerScheme.SCHEME_5_MIN,
            ACPowerScheme.SCHEME_10_MIN,
            ACPowerScheme.SCHEME_15_MIN,
            ACPowerScheme.SHCEME_30_MIN,
            ACPowerScheme.SCHEME_NEVER
        };

        BatteryPowerScheme[] BatteryPowerSchemeList = 
        {
            BatteryPowerScheme.SCHEME_1_MIN,
            BatteryPowerScheme.SCHEME_2_MIN,
            BatteryPowerScheme.SCHEME_3_MIN,
            BatteryPowerScheme.SCHEME_4_MIN,
            BatteryPowerScheme.SCHEME_5_MIN,
            BatteryPowerScheme.SCHEME_10_MIN,
            BatteryPowerScheme.SCHEME_15_MIN,
            BatteryPowerScheme.SCHEME_30_MIN,
            BatteryPowerScheme.SCHEME_NEVER
        };

        CPLProgram[] CPLProgramList =
        {
            CPLProgram.PC_Connection,
            CPLProgram.Dialing,
            CPLProgram.Keyboard,
            CPLProgram.Password,
            CPLProgram.Owner,
            CPLProgram.Power,
            CPLProgram.System,
            CPLProgram.Display,
            CPLProgram.Mouse,
            CPLProgram.Stylus,
            CPLProgram.Volume_Sounds,
            CPLProgram.InputPanel,
            CPLProgram.Remove_Programs,
            CPLProgram.Date_Time,
            CPLProgram.Certificates,
            CPLProgram.Bluetooth,
            CPLProgram.Net_Connections
        };

        #endregion

        public Form1()
        {
            InitializeComponent();

            DevInfo = new DeviceInfo();
            KBD_CTL = new KeypadControl();
            FnKeyCTL = new FunctionKeyControl();
            BKL_CTL = new BacklightControl();
            WLAN_CTL = new WLANControl();
            BT_CTL = new BluetoothControl();            
            SYS_CTL = new SystemControl();
            Boot_CTL = new Boot();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //  Device Information
            textBox_ModelName.Text = DevInfo.ModelName;
            textBox_FirmwareVersion.Text = DevInfo.FirmwareVersion;
            textBox_SerialNumber.Text = DevInfo.SerialNumber;
            textBox_LibraryVersion.Text = DevInfo.LibraryVersion;
            textBox_ConfigNum.Text = DevInfo.ConfigNumber;
            textBox_ROMSize.Text = DevInfo.ROMSize.ToString();
            textBox_RAMSize.Text = DevInfo.RAMSize.ToString();

            //  Keypad
            switch (KBD_CTL.InputMode)
            { 
                case KBDInputMode.CAPITAL_CHAR_MODE:
                    textBox_InputMode.Text = "Captial Char Mode";
                    break;
                case KBDInputMode.FUNCTION_MODE:
                    textBox_InputMode.Text = "Fn Mode";
                    break;
                case KBDInputMode.LOWERCASE_CHAR_MODE:
                    textBox_InputMode.Text = "Lowercase Char Mode";
                    break;
                case KBDInputMode.NUMERAL_MODE:
                    textBox_InputMode.Text = "Numeral Mode";
                    break;
            }
            
            int i=0;
            foreach (VKCode KeyCode in VKCodeList)
                comboBox_SelectKey.Items.Insert(i++, KeyCode.ToString());  
            comboBox_SelectKey.SelectedIndex = 0;

            bool lockState = new bool();
            KBD_CTL.GetKeypadState(VKCodeList[0], ref lockState);
            if (lockState)
                radioButton_KeyLocked.Checked = true;
            else
                radioButton_KeyUnlocked.Checked = true;

            //  Fnuction Key
            i = 0;
            foreach (FnVKCode KeyCode in FnVKCodeList)
                comboBox_SelectFnKey.Items.Insert(i++, KeyCode.ToString());
            comboBox_SelectFnKey.SelectedIndex = 0;
            comboBox_SelectFnKey_SelectedIndexChanged(this, e);

            //  Backlight
            i = 0;
            foreach (BKLMode mode in BKLModeList)
                comboBox_BKLMode.Items.Insert(i++, mode.ToString());

            i = 0;
            foreach (BKLTimer timer in BKLTimerList)
            {
                comboBox_BKLTimer.Items.Insert(i, timer.ToString().Remove(0, 6).Replace("_", " "));
                if (timer == BKL_CTL.Timer)
                    comboBox_BKLTimer.SelectedIndex = i;
                i++;
            }

            comboBox_BKL_Type.SelectedIndex = 0;
            comboBox1_SelectedIndexChanged(this, e);

            label_BKLValue.Text = BKL_CTL.Value.ToString() + "%";
            trackBar_BKLValue.Value = BKL_CTL.Value;

            //  Devices
            checkBox_WLANEnablePower.Checked = WLAN_CTL.EnablePower;
            textBox_WLANPHYAddress.Text = WLAN_CTL.PHYAddress;

            checkBox_BTEnablePower.Checked = BT_CTL.EnablePower;
            textBox_BT_PHYAddress.Text = BT_CTL.PHYAddress;

            int index = -1;
            switch (SYS_CTL.Vibrator)
            { 
                case VibratorState.DISABLE:
                    index = 0; break;
                case VibratorState.SHORT:
                    index = 1; break;
                case VibratorState.TWO_SHORT:
                    index = 2; break;
                case VibratorState.LONG:
                    index = 3; break;
            }
            comboBox1.SelectedIndex = index;

            //  Power
            comboBox_PowerMode.SelectedIndex = 0;
            comboBox_PowerMode_SelectedIndexChanged(this, e);

            comboBox_PowerType.SelectedIndex = 0;
            comboBox_PowerType_SelectedIndexChanged(this, e);

            //  Others
            textBox_ACLineState.Text = SYS_CTL.ACLineState.ToString();
            comboBox_SIPState.SelectedIndex = SYS_CTL.SIPState ? 0 : 1;

            i = 0;
            foreach (CPLProgram program in CPLProgramList)
                comboBox_LaunchProgram.Items.Insert(i++, program.ToString().Replace("_", " "));

            // Boot
            comboBox_BootMode.SelectedIndex = 0;

            int num = Boot_CTL.GetNumberOfBootProgram(BootMode.COLD_BOOT_INIT);
            textBox_NumOfBootProgram.Text = num.ToString();

            if (num > 0)
            {
                textBox_BootProgramPriority.Text = "0";
                textBox_BootProgramPath.Text = Boot_CTL.GetBootProgram(BootMode.COLD_BOOT_INIT, 0);
            }
            else
            {
                textBox_BootProgramPriority.Text = "";
                textBox_BootProgramPath.Text = "";
            }
        }

        // Keypad Tab
        private void button_KeyStatApply_Click(object sender, EventArgs e)
        {
            int index = comboBox_SelectKey.SelectedIndex;
            bool lockState = radioButton_KeyLocked.Checked? true:false;

            KBD_CTL.SetKeypadState(VKCodeList[index], lockState);
        }

        // FnKey Tab
        private void comboBox_SelectFnKey_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = comboBox_SelectFnKey.SelectedIndex;

            string FileName;
            string Parameters;

            FnKeyCTL.GetPrograms(FnVKCodeList[index], out FileName, out Parameters);
            textBox_FnKeyFilePath.Text = FileName;
            textBox_FnKeyParameter.Text = Parameters;
        }

        private void button_FnKeyApply_Click(object sender, EventArgs e)
        {
            int index = comboBox_SelectFnKey.SelectedIndex;

            FnKeyCTL.SetPrograms(FnVKCodeList[index], textBox_FnKeyFilePath.Text, textBox_FnKeyParameter.Text);
        }

        // Backlight Tab
        private void trackBar_BKLValue_ValueChanged(object sender, EventArgs e)
        {
            BKL_CTL.Value = trackBar_BKLValue.Value;
            label_BKLValue.Text = trackBar_BKLValue.Value.ToString() + "%";
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_BKL_Type.SelectedIndex == 0)
            {
                // Screen Backlight Control

                comboBox_BKLMode.Enabled = false;
                comboBox_BKLMode.SelectedIndex = -1;

                comboBox_BKLTimer.Enabled = true;
                int index;
                switch (BKL_CTL.Timer)
                {
                    case BKLTimer.TIMER_15_SEC:
                        index = 0; break;
                    case BKLTimer.TIMER_30_SEC:
                        index = 1; break;
                    case BKLTimer.TIMER_45_SEC:
                        index = 2; break;
                    case BKLTimer.TIMER_1_MIN:
                        index = 3; break;
                    case BKLTimer.TIMER_3_MIN:
                        index = 4; break;
                    case BKLTimer.TIMER_5_MIN:
                        index = 5; break;
                    case BKLTimer.TIMER_10_MIN:
                        index = 6; break;
                    case BKLTimer.TIMER_15_MIN:
                        index = 7; break;
                    default:
                        index = -1; break;
                }
                comboBox_BKLTimer.SelectedIndex = index;
            }
            else
            {
                // Keypad Backlight Control

                comboBox_BKLMode.Enabled = true;
                comboBox_BKLMode.SelectedIndex = (int)BKL_CTL.KeypadBKLMode;

                comboBox_BKLTimer.Enabled = false;
                comboBox_BKLTimer.SelectedIndex = -1;
            }
        }

        private void comboBox_BKLMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            BKL_CTL.KeypadBKLMode = (BKLMode)comboBox_BKLMode.SelectedIndex;
        }

        private void comboBox_BKLTimer_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = comboBox_BKLTimer.SelectedIndex;
            BKL_CTL.Timer = BKLTimerList[index];
        }

        // Devices Tab
        private void checkBox_WLANEnablePower_CheckStateChanged(object sender, EventArgs e)
        {
            WLAN_CTL.EnablePower = checkBox_WLANEnablePower.Checked;
        }

        private void checkBox_BTEnablePower_CheckStateChanged(object sender, EventArgs e)
        {
            BT_CTL.EnablePower = checkBox_BTEnablePower.Checked;
        }

        // Power Tab
        private void comboBox_PowerMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            comboBox_UserIdleTime.Items.Clear();
            comboBox_SystemIdleTime.Items.Clear();
            comboBox_SuspendTime.Items.Clear();

            if (comboBox_PowerMode.SelectedIndex == 1)
            {                                
                int index = 0;
                ACPowerScheme userIdleTime = SYS_CTL.AC_UserIdleTime;
                ACPowerScheme systemIdleTime = SYS_CTL.AC_SystemIdleTime;
                ACPowerScheme suspendTime = SYS_CTL.AC_SuspendTime;

                foreach (ACPowerScheme timer in ACPowerSchemeList)
                {
                    comboBox_UserIdleTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));                    
                    comboBox_SystemIdleTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));
                    comboBox_SuspendTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));

                    if (timer == userIdleTime)
                        comboBox_UserIdleTime.SelectedIndex = index;

                    if (timer == systemIdleTime)
                        comboBox_SystemIdleTime.SelectedIndex = index;

                    if (timer == suspendTime)
                        comboBox_SuspendTime.SelectedIndex = index;

                    index++;
                }
            }
            else
            {
                int index = 0;
                BatteryPowerScheme userIdleTime = SYS_CTL.Battery_UserIdleTime;
                BatteryPowerScheme systemIdleTime = SYS_CTL.Battery_SystemIdleTime;
                BatteryPowerScheme suspendTime = SYS_CTL.Battery_SuspendTime;

                foreach (BatteryPowerScheme timer in BatteryPowerSchemeList)
                {
                    comboBox_UserIdleTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));
                    comboBox_SystemIdleTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));
                    comboBox_SuspendTime.Items.Insert(index, timer.ToString().Remove(0, 7).Replace("_", " "));

                    if (timer == userIdleTime)
                        comboBox_UserIdleTime.SelectedIndex = index;
                    if (timer == systemIdleTime)
                        comboBox_SystemIdleTime.SelectedIndex = index;
                    if (timer == suspendTime)
                        comboBox_SuspendTime.SelectedIndex = index;

                    index++;
                }
            }
        }

        private void button_PowerSchemeApply_Click(object sender, EventArgs e)
        {
            if (comboBox_PowerMode.SelectedIndex == 1)
            {
                SYS_CTL.AC_UserIdleTime = ACPowerSchemeList[comboBox_UserIdleTime.SelectedIndex];
                SYS_CTL.AC_SystemIdleTime = ACPowerSchemeList[comboBox_SystemIdleTime.SelectedIndex];
                SYS_CTL.AC_SuspendTime = ACPowerSchemeList[comboBox_SuspendTime.SelectedIndex];                
            }
            else
            {
                SYS_CTL.Battery_UserIdleTime = BatteryPowerSchemeList[comboBox_UserIdleTime.SelectedIndex];
                SYS_CTL.Battery_SystemIdleTime = BatteryPowerSchemeList[comboBox_SystemIdleTime.SelectedIndex];
                SYS_CTL.Battery_SuspendTime = BatteryPowerSchemeList[comboBox_SuspendTime.SelectedIndex];
            }
        }

        private void comboBox_PowerType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_PowerType.SelectedIndex == 0)
            {
                textBox_PowerLevel.Text = SYS_CTL.MainBatteryPowerLevel.ToString().Replace("_", " ");
                textBox_ChargeState.Text = SYS_CTL.MainBatteryChargeState.ToString();
            }
            else
            {
                textBox_PowerLevel.Text = SYS_CTL.BackupBatteryPowerLevel.ToString().Replace("_", " ");
                textBox_ChargeState.Text = SYS_CTL.BackupBatteryChargeState.ToString();
            }
        }

        // Others Tab
        private void comboBox_SIPState_SelectedIndexChanged(object sender, EventArgs e)
        {
            SYS_CTL.SIPState = comboBox_SIPState.SelectedIndex == 0 ? true : false;
        }

        private void button_Launch_Click(object sender, EventArgs e)
        {
            SYS_CTL.LaunchCPLProgram((CPLProgram)comboBox_LaunchProgram.SelectedIndex);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SYS_CTL.WarmReset();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            SYS_CTL.StylusCalibration();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            SYS_CTL.StartActiveSync();
        }

        //Boot Tab
        private void comboBox_BootMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(comboBox_BootMode.SelectedIndex == -1)
                return;

            int mode = comboBox_BootMode.SelectedIndex;
            int num = Boot_CTL.GetNumberOfBootProgram((BootMode)mode);

            textBox_NumOfBootProgram.Text = num.ToString();

            if (num > 0)
            {
                textBox_BootProgramPriority.Text = "0";
                textBox_BootProgramPath.Text = Boot_CTL.GetBootProgram((BootMode)mode, 0);
            }
        }

        private void button_GetBootProgram_Click(object sender, EventArgs e)
        {
            BootMode mode = (BootMode)comboBox_BootMode.SelectedIndex;
            int priority = int.Parse(textBox_BootProgramPriority.Text);

            textBox_BootProgramPath.Text = Boot_CTL.GetBootProgram(mode, priority);
        }

        private void button_SetBootProgram_Click(object sender, EventArgs e)
        {
            if (comboBox_BootMode.SelectedIndex == -1 || textBox_BootProgramPriority.Text.Length == 0)
                return;

            BootMode mode = (BootMode)comboBox_BootMode.SelectedIndex;
            int priority = int.Parse(textBox_BootProgramPriority.Text);

            Boot_CTL.SetBootProgram(mode, priority, textBox_BootProgramPath.Text);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            SYS_CTL.TriggerVibration();
        }

        private void comboBox1_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            VibratorState state = VibratorState.DISABLE;

            switch (comboBox1.SelectedIndex)
            {
                case 0:
                    state = VibratorState.DISABLE; break;
                case 1:
                    state = VibratorState.SHORT; break;
                case 2:
                    state = VibratorState.TWO_SHORT; break;
                case 3:
                    state = VibratorState.LONG; break;
            }

            SYS_CTL.Vibrator = state;
        }

        

        

    }
}